@@ -1,12 +1,20 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
  * Copyright (C) 2003 Cordys R&D BV, All rights reserved.
  * Copyright (C) 2003-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
+/**
+ * SECTION:app
+ * @Short_description: Application support functions for Microsoft Crypto API.
+ * @Stability: Stable
+ *
+ */
+
 #include "globals.h"
 
 #include <string.h>
@@ -18,6 +26,7 @@
 #include <xmlsec/keys.h>
 #include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
+#include <xmlsec/keysdata.h>
 
 #include <xmlsec/mscrypto/app.h>
 #include <xmlsec/mscrypto/crypto.h>
@@ -479,10 +488,13 @@ xmlSecMSCryptoAppPkcs12LoadMemory(const xmlSecByte* data,
     PCCERT_CONTEXT tmpcert = NULL;
     PCCERT_CONTEXT pCert = NULL;
     WCHAR* wcPwd = NULL;
+    DWORD dwFlags;
     xmlSecKeyDataPtr x509Data = NULL;
     xmlSecKeyDataPtr keyData = NULL;
     xmlSecKeyPtr key = NULL;
-        int ret;
+    int ret;
+    DWORD dwData = 0;
+    DWORD dwDataLen;
 
     xmlSecAssert2(data != NULL, NULL);
     xmlSecAssert2(dataSize > 1, NULL);
@@ -509,7 +521,11 @@ xmlSecMSCryptoAppPkcs12LoadMemory(const xmlSecByte* data,
         goto done;
     }
 
-    hCertStore = PFXImportCertStore(&pfx, wcPwd, CRYPT_EXPORTABLE | PKCS12_NO_PERSIST_KEY);
+    dwFlags = CRYPT_EXPORTABLE;
+    if (!xmlSecImportGetPersistKey()) {
+        dwFlags |= PKCS12_NO_PERSIST_KEY;
+    }
+    hCertStore = PFXImportCertStore(&pfx, wcPwd, dwFlags);
     if (NULL == hCertStore) {
         xmlSecMSCryptoError("PFXImportCertStore", NULL);
         goto done;
@@ -523,12 +539,12 @@ xmlSecMSCryptoAppPkcs12LoadMemory(const xmlSecByte* data,
 
     while (1) {
         pCert = CertEnumCertificatesInStore(hCertStore, pCert);
-        if(pCert != NULL) {
+        if(pCert == NULL) {
             break;
         }
-        DWORD dwData = 0;
-        DWORD dwDataLen = sizeof(DWORD);
+        dwDataLen = sizeof(DWORD);
 
+        dwData = 0;
         /* Find the certificate that has the private key */
         if((TRUE == CertGetCertificateContextProperty(pCert, CERT_KEY_SPEC_PROP_ID, &dwData, &dwDataLen)) && (dwData > 0)) {
             tmpcert = CertDuplicateCertificateContext(pCert);
