@@ -1,5 +1,6 @@
-/**
- * XMLSec library
+/*
+ * XML Security Library (http://www.aleksey.com/xmlsec).
+ *
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
@@ -8,6 +9,13 @@
  * Copyright (C) 2003-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  * Copyright (c) 2005-2006 Cryptocom LTD (http://www.cryptocom.ru).
  */
+/**
+ * SECTION:crypto
+ * @Short_description: Crypto transforms implementation for Microsoft Crypto API.
+ * @Stability: Stable
+ *
+ */
+
 #include "globals.h"
 
 #include <string.h>
@@ -25,36 +33,14 @@
 #include <xmlsec/mscrypto/x509.h>
 #include "private.h"
 
-#if defined(__MINGW32__)
-/* NOTE mingw.org project don't define any xxx_s function and may
- * be never will define them.
- *
- * In this file is save to use non _s function as into destination
- * buffer program code copy empty string and the size of source buffer
- * (XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE=4096) is enough for any
- * encoding. Also program code don't check result of _s functions.
- */
-
-static int
-strcpy_s(char *dest, size_t n, const char *src) {
-    strcpy(dest, src);
-    return(0);
-}
-
-static int
-wcscpy_s(wchar_t *dest, size_t n, const wchar_t *src) {
-    wcscpy(dest, src);
-    return(0);
-}
-#endif
 
 #define XMLSEC_CONTAINER_NAME_A "xmlsec-key-container"
 #define XMLSEC_CONTAINER_NAME_W L"xmlsec-key-container"
 #ifdef UNICODE
 #define XMLSEC_CONTAINER_NAME XMLSEC_CONTAINER_NAME_W
-#else
+#else /* UNICODE */
 #define XMLSEC_CONTAINER_NAME XMLSEC_CONTAINER_NAME_A
-#endif
+#endif /* UNICODE */
 
 
 static xmlSecCryptoDLFunctionsPtr gXmlSecMSCryptoFunctions = NULL;
@@ -115,6 +101,11 @@ xmlSecCryptoGetFunctions_mscrypto(void) {
     gXmlSecMSCryptoFunctions->keyDataGost2001GetKlass           = xmlSecMSCryptoKeyDataGost2001GetKlass;
 #endif /* XMLSEC_NO_GOST*/
 
+#ifndef XMLSEC_NO_GOST2012
+    gXmlSecMSCryptoFunctions->keyDataGostR3410_2012_256GetKlass = xmlSecMSCryptoKeyDataGost2012_256GetKlass;
+    gXmlSecMSCryptoFunctions->keyDataGostR3410_2012_512GetKlass = xmlSecMSCryptoKeyDataGost2012_512GetKlass;
+#endif /* XMLSEC_NO_GOST2012*/
+
 #ifndef XMLSEC_NO_X509
     gXmlSecMSCryptoFunctions->keyDataX509GetKlass               = xmlSecMSCryptoKeyDataX509GetKlass;
     gXmlSecMSCryptoFunctions->keyDataRawX509CertGetKlass        = xmlSecMSCryptoKeyDataRawX509CertGetKlass;
@@ -161,6 +152,14 @@ xmlSecCryptoGetFunctions_mscrypto(void) {
     gXmlSecMSCryptoFunctions->transformGost2001GostR3411_94GetKlass             = xmlSecMSCryptoTransformGost2001GostR3411_94GetKlass;
 #endif /* XMLSEC_NO_GOST */
 
+#ifndef XMLSEC_NO_GOST2012
+    gXmlSecMSCryptoFunctions->transformGostR3411_2012_256GetKlass               = xmlSecMSCryptoTransformGostR3411_2012_256GetKlass;
+    gXmlSecMSCryptoFunctions->transformGostR3410_2012GostR3411_2012_256GetKlass = xmlSecMSCryptoTransformGost2012_256GetKlass;
+
+    gXmlSecMSCryptoFunctions->transformGostR3411_2012_512GetKlass               = xmlSecMSCryptoTransformGostR3411_2012_512GetKlass;
+    gXmlSecMSCryptoFunctions->transformGostR3410_2012GostR3411_2012_512GetKlass = xmlSecMSCryptoTransformGost2012_512GetKlass;
+#endif /* XMLSEC_NO_GOST2012 */
+
 #ifndef XMLSEC_NO_GOST
     gXmlSecMSCryptoFunctions->transformGostR3411_94GetKlass             = xmlSecMSCryptoTransformGostR3411_94GetKlass;
 #endif /* XMLSEC_NO_GOST */
@@ -273,11 +272,7 @@ int
 xmlSecMSCryptoInit (void)  {
     /* Check loaded xmlsec library version */
     if(xmlSecCheckVersionExact() != 1) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecCheckVersionExact",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecCheckVersionExact", NULL);
         return(-1);
     }
 
@@ -286,11 +281,7 @@ xmlSecMSCryptoInit (void)  {
 
     /* register our klasses */
     if(xmlSecCryptoDLFunctionsRegisterKeyDataAndTransforms(xmlSecCryptoGetFunctions_mscrypto()) < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecCryptoDLFunctionsRegisterKeyDataAndTransforms",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecCryptoDLFunctionsRegisterKeyDataAndTransforms", NULL);
         return(-1);
     }
     return(0);
@@ -330,21 +321,13 @@ xmlSecMSCryptoKeysMngrInit(xmlSecKeysMngrPtr mngr) {
 
         x509Store = xmlSecKeyDataStoreCreate(xmlSecMSCryptoX509StoreId);
         if(x509Store == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecKeyDataStoreCreate",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecMSCryptoX509StoreId");
+            xmlSecInternalError("xmlSecKeyDataStoreCreate(xmlSecMSCryptoX509StoreId)", NULL);
             return(-1);
         }
 
         ret = xmlSecKeysMngrAdoptDataStore(mngr, x509Store);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecKeysMngrAdoptDataStore",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        XMLSEC_ERRORS_NO_MESSAGE);
+            xmlSecInternalError("xmlSecKeysMngrAdoptDataStore", NULL);
             xmlSecKeyDataStoreDestroy(x509Store);
             return(-1);
         }
@@ -354,7 +337,6 @@ xmlSecMSCryptoKeysMngrInit(xmlSecKeysMngrPtr mngr) {
     return(0);
 }
 
-
 static xmlSecMSCryptoProviderInfo xmlSecMSCryptoProviderInfo_Random[] = {
     { MS_STRONG_PROV,               PROV_RSA_FULL },
     { MS_ENHANCED_PROV,             PROV_RSA_FULL },
@@ -381,29 +363,18 @@ xmlSecMSCryptoGenerateRandom(xmlSecBufferPtr buffer, size_t size) {
 
     ret = xmlSecBufferSetSize(buffer, size);
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecBufferSetSize",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "size=%d", size);
+        xmlSecInternalError2("xmlSecBufferSetSize", NULL,
+                             "size=%d", size);
         return(-1);
     }
 
     hProv = xmlSecMSCryptoFindProvider(xmlSecMSCryptoProviderInfo_Random, NULL, CRYPT_VERIFYCONTEXT, FALSE);
     if (0 == hProv) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecMSCryptoFindProvider",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecMSCryptoFindProvider", NULL);
         return(-1);
     }
     if (FALSE == CryptGenRandom(hProv, (DWORD)size, xmlSecBufferGetData(buffer))) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptGenRandom",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecMSCryptoError("CryptGenRandom", NULL);
         CryptReleaseContext(hProv,0);
         return(-1);
     }
@@ -412,7 +383,67 @@ xmlSecMSCryptoGenerateRandom(xmlSecBufferPtr buffer, size_t size) {
     return(0);
 }
 
-#define XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE       4096
+/**
+ * xmlSecMSCryptoGetErrorMessage:
+ * @dwError:            the error code.
+ * @out:                the output buffer.
+ * $outSize:            the output buffer size.
+ *
+ * Returns the system error message for the give error code.
+ */
+void
+xmlSecMSCryptoGetErrorMessage(DWORD dwError, xmlChar * out, xmlSecSize outSize) {
+    LPTSTR errorText = NULL;
+    DWORD ret;
+#ifndef UNICODE
+    WCHAR errorTextW[XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE];
+#endif /* UNICODE */
+
+    xmlSecAssert(out != NULL);
+    xmlSecAssert(outSize > 0);
+
+    /* Use system message tables to retrieve error text, allocate buffer on local
+       heap for error text, don't use any inserts/parameters */
+    ret = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM
+                      | FORMAT_MESSAGE_ALLOCATE_BUFFER
+                      | FORMAT_MESSAGE_IGNORE_INSERTS,
+                      NULL,
+                      dwError,
+                      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), /* Default language */
+                      (LPTSTR)&errorText,
+                      0,
+                      NULL);
+    if((ret <= 0) || (errorText == NULL)) {
+        out[0] = '\0';
+        goto done;
+    }
+
+#ifdef UNICODE
+    ret = WideCharToMultiByte(CP_UTF8, 0, errorText, -1, out, outSize, NULL, NULL);
+    if(ret <= 0) {
+        out[0] = '\0';
+        goto done;
+    }
+#else /* UNICODE */
+    ret = MultiByteToWideChar(CP_ACP, 0, errorText, -1, errorTextW, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE);
+    if(ret <= 0) {
+        out[0] = '\0';
+        goto done;
+    }
+    ret = WideCharToMultiByte(CP_UTF8, 0, errorTextW, -1, out, outSize, NULL, NULL);
+    if(ret <= 0) {
+        out[0] = '\0';
+        goto done;
+    }
+#endif /* UNICODE */
+
+done:
+    if(errorText != NULL) {
+        LocalFree(errorText);
+    }
+    return;
+}
+
 
 /**
  * xmlSecMSCryptoErrorsDefaultCallback:
@@ -424,59 +455,13 @@ xmlSecMSCryptoGenerateRandom(xmlSecBufferPtr buffer, size_t size) {
  * @reason:             the error code.
  * @msg:                the additional error message.
  *
- * The default errors reporting callback function.
+ * The default errors reporting callback function. Just a pass through to the default callback.
  */
 void
 xmlSecMSCryptoErrorsDefaultCallback(const char* file, int line, const char* func,
                                 const char* errorObject, const char* errorSubject,
                                 int reason, const char* msg) {
-    DWORD dwError;
-    TCHAR errorT[XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE];
-    WCHAR errorW[XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE];
-    CHAR  errorUTF8[XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE];
-    xmlChar buf[XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE];
-    DWORD rc;
-    int ret;
-
-    dwError = GetLastError();
-    rc = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
-                  NULL,
-                  dwError,
-                  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), /* Default language */
-                  errorT,
-                  XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE,
-                  NULL);
-    
-#ifdef UNICODE
-    if(rc <= 0) {
-        wcscpy_s(errorT, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE, L"");
-    }
-    ret = WideCharToMultiByte(CP_UTF8, 0, errorT, -1, errorUTF8, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE, NULL, NULL);
-    if(ret <= 0) {
-        strcpy_s(errorUTF8, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE, "");
-    }
-#else /* UNICODE */
-    if(rc <= 0) {
-        strcpy_s(errorT, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE, "");
-    }
-    ret = MultiByteToWideChar(CP_ACP, 0, errorT, -1, errorW, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE);
-    if(ret <= 0) {
-        wcscpy_s(errorW, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE, L"");
-    }
-    ret = WideCharToMultiByte(CP_UTF8, 0, errorW, -1, errorUTF8, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE, NULL, NULL);
-    if(ret <= 0) {
-        strcpy_s(errorUTF8, XMLSEC_MSCRYPTO_ERROR_MSG_BUFFER_SIZE, "");
-    }
-#endif /* UNICODE */
-
-    if((msg != NULL) && ((*msg) != '\0')) {
-        xmlSecStrPrintf(buf, sizeof(buf), BAD_CAST "%s;last error=%d (0x%08x);last error msg=%s", msg, dwError, dwError, errorUTF8);
-    } else {
-        xmlSecStrPrintf(buf, sizeof(buf), BAD_CAST "last error=%d (0x%08x);last error msg=%s", dwError, dwError, errorUTF8);
-    }
-    xmlSecErrorsDefaultCallback(file, line, func,
-                errorObject, errorSubject,
-                reason, (char*)buf);
+    xmlSecErrorsDefaultCallback(file, line, func, errorObject, errorSubject, reason, msg);
 }
 
 /**
@@ -496,7 +481,7 @@ xmlSecMSCryptoConvertUtf8ToUnicode(const xmlChar* str) {
     xmlSecAssert2(str != NULL, NULL);
 
     /* call MultiByteToWideChar first to get the buffer size */
-    ret = MultiByteToWideChar(CP_UTF8, 0, str, -1, NULL, 0);
+    ret = MultiByteToWideChar(CP_UTF8, 0, (LPCCH)str, -1, NULL, 0);
     if(ret <= 0) {
         return(NULL);
     }
@@ -505,16 +490,12 @@ xmlSecMSCryptoConvertUtf8ToUnicode(const xmlChar* str) {
     /* allocate buffer */
     res = (LPWSTR)xmlMalloc(sizeof(WCHAR) * len);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", sizeof(WCHAR) * len);
+        xmlSecMallocError(sizeof(WCHAR) * len, NULL);
         return(NULL);
     }
 
     /* convert */
-    ret = MultiByteToWideChar(CP_UTF8, 0, str, -1, res, len);
+    ret = MultiByteToWideChar(CP_UTF8, 0, (LPCCH)str, -1, res, len);
     if(ret <= 0) {
         xmlFree(res);
         return(NULL);
@@ -550,16 +531,12 @@ xmlSecMSCryptoConvertUnicodeToUtf8(LPCWSTR str) {
     /* allocate buffer */
     res = (xmlChar*)xmlMalloc(sizeof(xmlChar) * len);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", sizeof(xmlChar) * len);
+        xmlSecMallocError(sizeof(xmlChar) * len, NULL);
         return(NULL);
     }
 
     /* convert */
-    ret = WideCharToMultiByte(CP_UTF8, 0, str, -1, res, len, NULL, NULL);
+    ret = WideCharToMultiByte(CP_UTF8, 0, str, -1, (LPSTR)res, len, NULL, NULL);
     if(ret <= 0) {
         xmlFree(res);
         return(NULL);
@@ -595,19 +572,15 @@ xmlSecMSCryptoConvertLocaleToUnicode(const char* str) {
     /* allocate buffer */
     res = (LPWSTR)xmlMalloc(sizeof(WCHAR) * len);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecMallocError(sizeof(WCHAR) * len, NULL);
         return(NULL);
     }
 
     /* convert */
     ret = MultiByteToWideChar(CP_ACP, 0, str, -1, res, len);
     if(ret <= 0) {
-            xmlFree(res);
-            return(NULL);
+        xmlFree(res);
+        return(NULL);
     }
 
     /* done */
@@ -647,17 +620,13 @@ xmlSecMSCryptoConvertLocaleToUtf8(const char * str) {
     /* allocate buffer */
     res = (xmlChar*)xmlMalloc(sizeof(xmlChar) * len);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", sizeof(xmlChar) * len);
+        xmlSecMallocError(sizeof(xmlChar) * len, NULL);
         xmlFree(strW);
         return(NULL);
     }
 
     /* convert */
-    ret = WideCharToMultiByte(CP_ACP, 0, strW, -1, res, len, NULL, NULL);
+    ret = WideCharToMultiByte(CP_ACP, 0, strW, -1, (LPSTR)res, len, NULL, NULL);
     if(ret <= 0) {
         xmlFree(strW);
         xmlFree(res);
@@ -702,11 +671,7 @@ xmlSecMSCryptoConvertUtf8ToLocale(const xmlChar* str) {
     /* allocate buffer */
     res = (char*)xmlMalloc(sizeof(char) * len);
     if(res == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "size=%d", sizeof(xmlChar) * len);
+        xmlSecMallocError(sizeof(char) * len, NULL);
         xmlFree(strW);
         return(NULL);
     }
