@@ -1,14 +1,19 @@
-/**
+/*
+ * XML Security Library (http://www.aleksey.com/xmlsec).
  *
- * XMLSec library
- *
- * DES Algorithm support
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
  * Copyright (C) 2003 Cordys R&D BV, All rights reserved.
  */
+/**
+ * SECTION:symkeys
+ * @Short_description: Symmetric keys implementation for Microsoft Crypto API.
+ * @Stability: Private
+ *
+ */
+
 #include "globals.h"
 
 #include <stdlib.h>
@@ -238,40 +243,24 @@ xmlSecMSCryptoCreatePrivateExponentOneKey(HCRYPTPROV hProv, HCRYPTKEY *hPrivateK
 
     /* Generate the private key */
     if(!CryptGenKey(hProv, AT_KEYEXCHANGE, CRYPT_EXPORTABLE, &hKey)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptGenKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecMSCryptoError("CryptGenKey", NULL);
         goto done;
     }
 
     /* Export the private key, we'll convert it to a private exponent of one key */
     if(!CryptExportKey(hKey, 0, PRIVATEKEYBLOB, 0, NULL, &keyBlobLen)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptExportKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecMSCryptoError("CryptExportKey", NULL);
         goto done;
     }
 
     keyBlob = (LPBYTE)xmlMalloc(sizeof(BYTE) * keyBlobLen);
     if(keyBlob == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecMallocError(sizeof(BYTE) * keyBlobLen, NULL);
         goto done;
     }
 
     if(!CryptExportKey(hKey, 0, PRIVATEKEYBLOB, 0, keyBlob, &keyBlobLen)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptExportKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecMSCryptoError("CryptExportKey", NULL);
         goto done;
     }
     CryptDestroyKey(hKey);
@@ -279,28 +268,22 @@ xmlSecMSCryptoCreatePrivateExponentOneKey(HCRYPTPROV hProv, HCRYPTKEY *hPrivateK
 
     /* Get the bit length of the key */
     if(keyBlobLen < sizeof(PUBLICKEYSTRUC) + sizeof(RSAPUBKEY)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptExportKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "len=%ld", keyBlobLen);
+        xmlSecMSCryptoError2("CryptExportKey", NULL,
+                             "len=%ld",
+                             (long int)keyBlobLen);
         goto done;
     }
     pubKeyStruc = (PUBLICKEYSTRUC*)keyBlob;
     if(pubKeyStruc->bVersion != 0x02) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptExportKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "pubKeyStruc->bVersion=%d", pubKeyStruc->bVersion);
+        xmlSecMSCryptoError2("CryptExportKey", NULL,
+                             "pubKeyStruc->bVersion=%ld",
+                             (long int)pubKeyStruc->bVersion);
         goto done;
     }
     if(pubKeyStruc->bType != PRIVATEKEYBLOB) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptExportKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "pubKeyStruc->bType=%d", (int)pubKeyStruc->bType);
+        xmlSecMSCryptoError2("CryptExportKey", NULL,
+                             "pubKeyStruc->bType=%ld",
+                             (long int)pubKeyStruc->bType);
         goto done;
     }
 
@@ -309,11 +292,9 @@ xmlSecMSCryptoCreatePrivateExponentOneKey(HCRYPTPROV hProv, HCRYPTKEY *hPrivateK
 
     /* check that we have RSA private key */
     if(rsaPubKey->magic != 0x32415352) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptExportKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "rsaPubKey->magic=0x%08lx", rsaPubKey->magic);
+        xmlSecMSCryptoError2("CryptExportKey", NULL,
+                             "rsaPubKey->magic=0x%08lx",
+                             (long int)rsaPubKey->magic);
         goto done;
     }
     bitLen = rsaPubKey->bitlen;
@@ -335,11 +316,8 @@ xmlSecMSCryptoCreatePrivateExponentOneKey(HCRYPTPROV hProv, HCRYPTKEY *hPrivateK
      * BYTE privateExponent[rsapubkey.bitlen/8];        1/8
      */
     if(keyBlobLen < sizeof(PUBLICKEYSTRUC) + sizeof(RSAPUBKEY) + bitLen / 2 + bitLen / 16) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptExportKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "len=%ld", keyBlobLen);
+        xmlSecMSCryptoError2("CryptExportKey", NULL,
+                             "keBlobLen=%ld", keyBlobLen);
         goto done;
     }
     ptr = (BYTE*)(keyBlob + sizeof(PUBLICKEYSTRUC) + sizeof(RSAPUBKEY));
@@ -374,11 +352,7 @@ xmlSecMSCryptoCreatePrivateExponentOneKey(HCRYPTPROV hProv, HCRYPTKEY *hPrivateK
 
     /* Import the exponent-of-one private key. */
     if (!CryptImportKey(hProv, keyBlob, keyBlobLen, 0, 0, &hKey)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptImportKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecMSCryptoError("CryptImportKey", NULL);
         goto done;
     }
     (*hPrivateKey) = hKey;
@@ -434,11 +408,9 @@ xmlSecMSCryptoImportPlainSessionBlob(HCRYPTPROV hProv, HCRYPTKEY hPrivateKey,
         dwFlags = 0;
     }
     if(!fFound) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptGetProvParam",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "algId=%d is not supported", dwAlgId);
+        xmlSecMSCryptoError2("CryptGetProvParam", NULL,
+                             "algId=%ld is not supported",
+                             (long int)dwAlgId);
         goto done;
     }
 
@@ -447,21 +419,16 @@ xmlSecMSCryptoImportPlainSessionBlob(HCRYPTPROV hProv, HCRYPTKEY hPrivateKey,
          * PP_ENUMALGS_EX contains the key size without the padding so we can't use it.
          */
         if(!CryptGenKey(hProv, dwAlgId, 0, &hTempKey)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "CryptGenKey",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        "algId=%d", dwAlgId);
+            xmlSecMSCryptoError2("CryptGenKey", NULL,
+                                 "algId=%ld",
+                                 (long int)dwAlgId);
             goto done;
         }
 
         dwSize = sizeof(DWORD);
         if(!CryptGetKeyParam(hTempKey, KP_KEYLEN, (LPBYTE)&dwProvSessionKeySize, &dwSize, 0)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "CryptGetKeyParam(KP_KEYLEN)",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        "algId=%d", dwAlgId);
+            xmlSecMSCryptoError2("CryptGetKeyParam(KP_KEYLEN)", NULL,
+                                 "algId=%ld", (long int)dwAlgId);
             goto done;
         }
         CryptDestroyKey(hTempKey);
@@ -469,12 +436,9 @@ xmlSecMSCryptoImportPlainSessionBlob(HCRYPTPROV hProv, HCRYPTKEY hPrivateKey,
 
         /* yell if key is too big */
         if ((dwKeyMaterial * 8) > dwProvSessionKeySize) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        NULL,
-                        XMLSEC_ERRORS_R_INVALID_SIZE,
-                        "dwKeyMaterial=%ld;dwProvSessionKeySize=%ld",
-                        dwKeyMaterial, dwProvSessionKeySize);
+            xmlSecInvalidSizeMoreThanError("Key value (bits)",
+                                           (dwKeyMaterial * 8), dwProvSessionKeySize,
+                                           NULL);
             goto done;
         }
     } else {
@@ -484,33 +448,24 @@ xmlSecMSCryptoImportPlainSessionBlob(HCRYPTPROV hProv, HCRYPTKEY hPrivateKey,
     /* Get private key's algorithm */
     dwSize = sizeof(ALG_ID);
     if(!CryptGetKeyParam(hPrivateKey, KP_ALGID, (LPBYTE)&dwPrivKeyAlg, &dwSize, 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptGetKeyParam(KP_ALGID)",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "algId=%d", dwAlgId);
+        xmlSecMSCryptoError2("CryptGetKeyParam(KP_ALGID)", NULL,
+                             "algId=%ld",
+                             (long int)dwAlgId);
         goto done;
     }
 
     /* Get private key's length in bits */
     dwSize = sizeof(DWORD);
     if(!CryptGetKeyParam(hPrivateKey, KP_KEYLEN, (LPBYTE)&dwPublicKeySize, &dwSize, 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptGetKeyParam(KP_KEYLEN)",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "algId=%d", dwAlgId);
+        xmlSecMSCryptoError2("CryptGetKeyParam(KP_KEYLEN)", NULL,
+                             "algId=%ld",
+                             (long int)dwAlgId);
         goto done;
     }
 
     /* 3 is for the first reserved byte after the key material and the 2 reserved bytes at the end. */
     if(dwPublicKeySize / 8 < dwKeyMaterial + 3) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_INVALID_SIZE,
-                    "dwKeyMaterial=%ld;dwPublicKeySize=%ld",
-                    dwKeyMaterial, dwPublicKeySize);
+        xmlSecInvalidSizeLessThanError("Key value", dwPublicKeySize / 8, dwKeyMaterial + 3, NULL);
         goto done;
     }
     rndBlobSize = dwPublicKeySize / 8 - (dwKeyMaterial + 3);
@@ -530,11 +485,7 @@ xmlSecMSCryptoImportPlainSessionBlob(HCRYPTPROV hProv, HCRYPTKEY hPrivateKey,
     /* allocate simple blob buffer */
     keyBlob = (LPBYTE)xmlMalloc(sizeof(BYTE) * keyBlobLen);
     if(keyBlob == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecMallocError(sizeof(BYTE) * keyBlobLen, NULL);
         goto done;
     }
     memset(keyBlob, 0, keyBlobLen);
@@ -562,11 +513,9 @@ xmlSecMSCryptoImportPlainSessionBlob(HCRYPTPROV hProv, HCRYPTKEY hPrivateKey,
 
     /* Generate random data for the rest of the buffer */
     if((rndBlobSize > 0) && !CryptGenRandom(hProv, rndBlobSize, pbPtr)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptGenRandom",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "rndBlobSize=%ld", rndBlobSize);
+        xmlSecMSCryptoError2("CryptGenRandom", NULL,
+                             "rndBlobSize=%ld",
+                             (long int)rndBlobSize);
         goto done;
     }
     /* aleksey: why are we doing this? */
@@ -578,11 +527,9 @@ xmlSecMSCryptoImportPlainSessionBlob(HCRYPTPROV hProv, HCRYPTKEY hPrivateKey,
     keyBlob[keyBlobLen - 2] = 2;
 
     if(!CryptImportKey(hProv, keyBlob , keyBlobLen, hPrivateKey, CRYPT_EXPORTABLE, hSessionKey)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "CryptImportKey",
-                    XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                    "algId=%d", dwAlgId);
+        xmlSecMSCryptoError2("CryptImportKey", NULL,
+                             "algId=%ld",
+                             (long int)dwAlgId);
         goto done;
     }
 
