@@ -1,13 +1,19 @@
-/**
+/*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
- * Enchanced nodes set
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
+/**
+ * SECTION:nodeset
+ * @Short_description: XML nodes set functions
+ * @Stability: Stable
+ *
+ */
+
 #include "globals.h"
 
 #include <stdlib.h>
@@ -41,7 +47,7 @@ static int      xmlSecNodeSetWalkRecursive              (xmlSecNodeSetPtr nset,
  * @nodes:              the list of nodes.
  * @type:               the nodes set type.
  *
- * Creates new nodes set. Caller is responsible for freeng returend object
+ * Creates new nodes set. Caller is responsible for freeing returned object
  * by calling #xmlSecNodeSetDestroy function.
  *
  * Returns: pointer to newly allocated node set or NULL if an error occurs.
@@ -52,12 +58,7 @@ xmlSecNodeSetCreate(xmlDocPtr doc, xmlNodeSetPtr nodes, xmlSecNodeSetType type)
 
     nset = (xmlSecNodeSetPtr)xmlMalloc(sizeof(xmlSecNodeSet));
     if(nset == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_MALLOC_FAILED,
-                    "sizeof(xmlSecNodeSet)=%d",
-                    sizeof(xmlSecNodeSet));
+        xmlSecMallocError(sizeof(xmlSecNodeSet), NULL);
         return(NULL);
     }
     memset(nset, 0,  sizeof(xmlSecNodeSet));
@@ -193,11 +194,8 @@ xmlSecNodeSetOneContains(xmlSecNodeSetPtr nset, xmlNodePtr node, xmlNodePtr pare
         }
         return(1);
     default:
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_INVALID_TYPE,
-                    "type=%d", nset->type);
+        xmlSecInvalidIntegerTypeError("node set type", nset->type,
+                "supported nodeset type", NULL);
     }
 
     return(0);
@@ -246,11 +244,8 @@ xmlSecNodeSetContains(xmlSecNodeSetPtr nset, xmlNodePtr node, xmlNodePtr parent)
             }
             break;
         default:
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        NULL,
-                        XMLSEC_ERRORS_R_INVALID_OPERATION,
-                        "operation=%d", cur->op);
+            xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_OPERATION, NULL,
+                              "node set operation=%d", (int)cur->op);
             return(-1);
         }
         cur = cur->next;
@@ -310,22 +305,14 @@ xmlSecNodeSetAddList(xmlSecNodeSetPtr nset, xmlSecNodeSetPtr newNSet, xmlSecNode
 
     tmp1 = xmlSecNodeSetCreate(newNSet->doc, NULL, xmlSecNodeSetList);
     if(tmp1 == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecNodeSetCreate",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecNodeSetCreate", NULL);
         return(NULL);
     }
     tmp1->children = newNSet;
 
     tmp2 = xmlSecNodeSetAdd(nset, tmp1, op);
     if(tmp2 == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecNodeSetAdd",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecNodeSetAdd", NULL);
         xmlSecNodeSetDestroy(tmp1);
         return(NULL);
     }
@@ -477,11 +464,7 @@ xmlSecNodeSetGetChildren(xmlDocPtr doc, const xmlNodePtr parent, int withComment
 
     nodes = xmlXPathNodeSetCreate(parent);
     if(nodes == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlXPathNodeSetCreate",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlXPathNodeSetCreate", NULL);
         return(NULL);
     }
 
@@ -512,13 +495,19 @@ static int
 xmlSecNodeSetDumpTextNodesWalkCallback(xmlSecNodeSetPtr nset, xmlNodePtr cur,
                                    xmlNodePtr parent ATTRIBUTE_UNUSED,
                                    void* data) {
+    int ret;
     xmlSecAssert2(nset != NULL, -1);
     xmlSecAssert2(cur != NULL, -1);
     xmlSecAssert2(data != NULL, -1);
 
-    if(cur->type == XML_TEXT_NODE) {
-        xmlOutputBufferWriteString((xmlOutputBufferPtr)data,
-                                    (char*)(cur->content));
+    if(cur->type != XML_TEXT_NODE) {
+        return(0);
+    }
+    ret = xmlOutputBufferWriteString((xmlOutputBufferPtr)data,
+            (char*)(cur->content));
+    if(ret < 0) {
+        xmlSecXmlError("xmlOutputBufferWriteString", NULL);
+        return(-1);
     }
     return(0);
 }
@@ -583,11 +572,8 @@ xmlSecNodeSetDebugDump(xmlSecNodeSetPtr nset, FILE *output) {
         return;
     default:
         fprintf(output, "(unknown=%d)\n", nset->type);
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    NULL,
-                    XMLSEC_ERRORS_R_INVALID_TYPE,
-                    "type=%d", nset->type);
+        xmlSecInvalidIntegerTypeError("node set type", nset->type,
+                "supported nodeset type", NULL);
     }
 
     l = xmlXPathNodeSetGetLength(nset->nodes);
