@@ -21,7 +21,7 @@ var baseName = "libxmlsec";
 
 /* Configure file which contains the version and the output file where
    we can store our build configuration. */
-var configFile = baseDir + "\\configure.in";
+var configFile = baseDir + "\\configure.ac";
 var versionFile = ".\\configure.txt";
 
 /* Input and output files regarding the lib(e)xml features. The second
@@ -30,6 +30,10 @@ var versionFile = ".\\configure.txt";
 var optsFileIn = baseDir + "\\config.h.in";
 var optsFile = baseDir + "\\config.h";
 
+/* Input and output files regarding the xmlsec version. */
+var versionHeaderIn = baseDir + "\\include\\xmlsec\\version.h.in";
+var versionHeader = baseDir + "\\include\\xmlsec\\version.h";
+
 /* Version strings for the binary distribution. Will be filled later 
    in the code. */
 var verMajorXmlSec;
@@ -43,6 +47,7 @@ var withOpenSSL = 0;
 var withOpenSSLVersion = "";
 var withNss = 0;
 var withMSCrypto = 0;
+var withMSCng = 0;
 var withLibXSLT = 1;
 var withIconv = 1;
 var withNT4 = 1;
@@ -50,6 +55,8 @@ var withNT4 = 1;
 /* Win32 build options. */
 var buildUnicode = 1;
 var buildDebug = 0;
+var buildWerror = 0;
+var buildCc = "cl.exe";
 var buildStatic = 1;
 var buildWithDLSupport = 1;
 var buildPrefix = ".";
@@ -59,6 +66,8 @@ var buildLibPrefix = "$(PREFIX)\\lib";
 var buildSoPrefix = "$(PREFIX)\\lib";
 var buildInclude = ".";
 var buildLib = ".";
+var cruntime = "/MD";
+
 /* Local stuff */
 var error = 0;
 
@@ -97,14 +106,16 @@ function usage()
 	txt += "either 'yes' or 'no'.\n\n";
 	txt += "XmlSec Library options, default value given in parentheses:\n\n";
 	txt += "  crypto:     Crypto engines list, first is default: \"openssl\",\n";
-	txt += "              \"openssl=096\", \"openssl=097\", \"openssl=098\", \n";
-	txt += "              \"nss\", \"mscrypto\" (\"" + withCrypto + "\");\n"
+	txt += "              \"openssl=100\", \"openssl=110\", \n";
+	txt += "              \"nss\", \"mscrypto\", \"mscng\" (\"" + withCrypto + "\");\n"
  	txt += "  xslt:       LibXSLT is used (" + (withLibXSLT? "yes" : "no")  + ")\n";	
  	txt += "  iconv:      Use the iconv library (" + (withIconv? "yes" : "no")  + ")\n";	
  	txt += "  nt4:        Enable NT 4.0 support (" + (withNT4? "yes" : "no")  + ")\n";	
 	txt += "\nWin32 build options, default value given in parentheses:\n\n";
 	txt += "  unicode:    Build Unicode version (" + (buildUnicode? "yes" : "no")  + ")\n";
 	txt += "  debug:      Build unoptimised debug executables (" + (buildDebug? "yes" : "no")  + ")\n";
+	txt += "  werror:     Build with warnings as errors(" + (buildWerror? "yes" : "no")  + ")\n";
+	txt += "  cc:         Build with the specified compiler(" + buildCc  + ")\n";
 	txt += "  static:     Link libxmlsec statically to xmlsec (" + (buildStatic? "yes" : "no")  + ")\n";
 	txt += "  with-dl:    Enable dynamic loading of xmlsec-crypto libraries (" + (buildWithDLSupport? "yes" : "no")  + ")\n";
 	txt += "  prefix:     Base directory for the installation (" + buildPrefix + ")\n";
@@ -159,14 +170,17 @@ function discoverVersion()
 	vf.WriteLine("WITH_CRYPTO=" + withCrypto);	
 	vf.WriteLine("WITH_DEFAULT_CRYPTO=" + withDefaultCrypto);	
 	vf.WriteLine("WITH_OPENSSL=" + withOpenSSL);	
-	vf.WriteLine("WITH_OPENSSL_VERSION=XMLSEC_OPENSSL_" + withOpenSSLVersion);	
+	vf.WriteLine("WITH_OPENSSL_VERSION=XMLSEC_OPENSSL_" + withOpenSSLVersion);
 	vf.WriteLine("WITH_NSS=" + withNss);	
 	vf.WriteLine("WITH_MSCRYPTO=" + withMSCrypto);	
+	vf.WriteLine("WITH_MSCNG=" + withMSCng);
 	vf.WriteLine("WITH_LIBXSLT=" + (withLibXSLT ? "1" : "0"));
 	vf.WriteLine("WITH_ICONV=" + (withIconv ? "1" : "0"));
 	vf.WriteLine("WITH_NT4=" + (withNT4 ? "1" : "0"));
 	vf.WriteLine("UNICODE=" + (buildUnicode? "1" : "0"));
 	vf.WriteLine("DEBUG=" + (buildDebug? "1" : "0"));
+	vf.WriteLine("WERROR=" + (buildWerror? "1" : "0"));
+	vf.WriteLine("CC=" + buildCc);
 	vf.WriteLine("STATIC=" + (buildStatic? "1" : "0"));
 	vf.WriteLine("WITH_DL=" + (buildWithDLSupport ? "1" : "0"));
 	vf.WriteLine("PREFIX=" + buildPrefix);
@@ -176,6 +190,7 @@ function discoverVersion()
 	vf.WriteLine("SOPREFIX=" + buildSoPrefix);
 	vf.WriteLine("INCLUDE=$(INCLUDE);" + buildInclude);
 	vf.WriteLine("LIB=$(LIB);" + buildLib);
+	vf.WriteLine("CRUNTIME=" + cruntime);
 	vf.Close();
 }
 
@@ -203,6 +218,43 @@ function configureXmlSec()
 	of.Close();
 }
 
+/* This one will generate version.h from version.h.in. */
+function configureXmlSecVersion()
+{
+	var fso, ofi, of, ln, s;
+	fso = new ActiveXObject("Scripting.FileSystemObject");
+	if (fso.FileExists(versionHeader)) {
+		// version.h is already generated, nothing to do.
+		return;
+	}
+
+	ofi = fso.OpenTextFile(versionHeaderIn, 1);
+	of = fso.CreateTextFile(versionHeader, true);
+	while (ofi.AtEndOfStream != true) {
+		ln = ofi.ReadLine();
+		s = new String(ln);
+		if (s.search(/\@XMLSEC_VERSION_MAJOR\@/) != -1) {
+			of.WriteLine(s.replace(/\@XMLSEC_VERSION_MAJOR\@/,
+				verMajorXmlSec));
+		} else if (s.search(/\@XMLSEC_VERSION_MINOR\@/) != -1) {
+			of.WriteLine(s.replace(/\@XMLSEC_VERSION_MINOR\@/,
+				verMinorXmlSec));
+		} else if (s.search(/\@XMLSEC_VERSION_SUBMINOR\@/) != -1) {
+			of.WriteLine(s.replace(/\@XMLSEC_VERSION_SUBMINOR\@/,
+				verMicroXmlSec));
+		} else if (s.search(/\@XMLSEC_VERSION\@/) != -1) {
+			of.WriteLine(s.replace(/\@XMLSEC_VERSION\@/,
+				verMajorXmlSec + "." + verMinorXmlSec + "." + verMicroXmlSec));
+		} else if (s.search(/\@XMLSEC_VERSION_INFO\@/) != -1) {
+			of.WriteLine(s.replace(/\@XMLSEC_VERSION_INFO\@/,
+				(parseInt(verMajorXmlSec) + parseInt(verMinorXmlSec)) + ":" + verMicroXmlSec + ":" + verMinorXmlSec));
+		} else
+			of.WriteLine(ln);
+	}
+	ofi.Close();
+	of.Close();
+}
+
 /* Creates the readme file for the binary distribution of 'bname', for the
    version 'ver' in the file 'file'. This one is called from the Makefile when
    generating a binary distribution. The parameters are passed by make. */
@@ -261,6 +313,10 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 			buildUnicode = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "debug")
 			buildDebug = strToBool(arg.substring(opt.length + 1, arg.length));
+		else if (opt == "werror")
+			buildWerror = strToBool(arg.substring(opt.length + 1, arg.length));
+		else if (opt == "cc")
+			buildCc = arg.substring(opt.length + 1, arg.length);
 		else if (opt == "static")
 			buildStatic = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "with-dl")
@@ -281,6 +337,8 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 			buildInclude = arg.substring(opt.length + 1, arg.length);
 		else if (opt == "lib")
 			buildLib = arg.substring(opt.length + 1, arg.length);
+		else if (opt == "cruntime")
+			cruntime = arg.substring(opt.length + 1, arg.length);
 		else
 			error = 1;
 	} else if (i == 0) {
@@ -311,25 +369,24 @@ for (j = 0; j < crlist.length; j++) {
 	if (crlist[j] == "openssl") {
 		curcrypto="openssl";
 		withOpenSSL = 1;
-		withOpenSSLVersion = "098"; /* default */
-	} else if (crlist[j] == "openssl=096") {
+		withOpenSSLVersion = "110"; /* default */
+	} else if (crlist[j] == "openssl=100") {
 		curcrypto="openssl";
 		withOpenSSL = 1;
-		withOpenSSLVersion = "096";
-	} else if (crlist[j] == "openssl=097") {
+		withOpenSSLVersion = "100";
+	} else if (crlist[j] == "openssl=110") {
 		curcrypto="openssl";
 		withOpenSSL = 1;
-		withOpenSSLVersion = "097";
-	} else if (crlist[j] == "openssl=098") {
-		curcrypto="openssl";
-		withOpenSSL = 1;
-		withOpenSSLVersion = "098";
+		withOpenSSLVersion = "110";
 	} else if (crlist[j] == "nss") {
 		curcrypto="nss";
 		withNss = 1;
 	} else if (crlist[j] == "mscrypto") {
 		curcrypto="mscrypto";
 		withMSCrypto = 1;
+	} else if (crlist[j] == "mscng") {
+		curcrypto="mscng";
+		withMSCng = 1;
 	} else {
 		WScript.Echo("Unknown crypto engine \"" + crlist[j] + "\" is found. Aborting.");
 		WScript.Quit(error);
@@ -353,6 +410,8 @@ WScript.Echo(baseName + " version: " + verMajorXmlSec + "." + verMinorXmlSec + "
 
 // Configure libxmlsec.
 configureXmlSec();
+// Generate version.h.
+configureXmlSecVersion();
 if (error != 0) {
 	WScript.Echo("Configuration failed, aborting.");
 	WScript.Quit(error);
@@ -373,14 +432,18 @@ txtOut += "        Use OpenSSL: " + boolToStr(withOpenSSL) + "\n";
 txtOut += "Use OpenSSL Version: " + withOpenSSLVersion + "\n";
 txtOut += "            Use NSS: " + boolToStr(withNss) + "\n";
 txtOut += "       Use MSCrypto: " + boolToStr(withMSCrypto) + "\n";
+txtOut += "          Use MSCng: " + boolToStr(withMSCng) + "\n";
 txtOut += "        Use LibXSLT: " + boolToStr(withLibXSLT) + "\n";
 txtOut += "          Use iconv: " + boolToStr(withIconv) + "\n";
 txtOut += "     NT 4.0 support: " + boolToStr(withNT4) + "\n";
 txtOut += "\n";
 txtOut += "Win32 build configuration\n";
 txtOut += "-------------------------\n";
+txtOut += "  C-Runtime option: " + cruntime + "\n";
 txtOut += "           Unicode: " + boolToStr(buildUnicode) + "\n";
 txtOut += "     Debug symbols: " + boolToStr(buildDebug) + "\n";
+txtOut += "Warnings as errors: " + boolToStr(buildWerror) + "\n";
+txtOut += "        C compiler: " + buildCc + "\n";
 txtOut += "     Static xmlsec: " + boolToStr(buildStatic) + "\n";
 txtOut += " Enable DL support: " + boolToStr(buildWithDLSupport) + "\n";
 txtOut += "    Install prefix: " + buildPrefix + "\n";
